@@ -68,7 +68,7 @@ orientation_change (AppletInfo  *info,
 	orientation = panel_widget_get_applet_orientation (panel);
 
 	switch (info->type) {
-	case PANEL_OBJECT_BONOBO:
+	case PANEL_OBJECT_APPLET:
 		panel_applet_frame_change_orientation (
 				PANEL_APPLET_FRAME (info->widget), orientation);
 		break;
@@ -128,7 +128,7 @@ void
 size_change (AppletInfo  *info,
 	     PanelWidget *panel)
 {
-	if (info->type == PANEL_OBJECT_BONOBO)
+	if (info->type == PANEL_OBJECT_APPLET)
 		panel_applet_frame_change_size (
 			PANEL_APPLET_FRAME (info->widget), panel->sz);
 }
@@ -155,7 +155,7 @@ back_change (AppletInfo  *info,
 	     PanelWidget *panel)
 {
 	switch (info->type) {
-	case PANEL_OBJECT_BONOBO:
+	case PANEL_OBJECT_APPLET:
 		panel_applet_frame_change_background (
 			PANEL_APPLET_FRAME (info->widget), panel->background.type);
 		break;
@@ -391,7 +391,7 @@ panel_key_press_event (GtkWidget   *widget,
 	 * bindings do not work. We get around this by
 	 * activating the key bindings here.
 	 */ 
-	if (GTK_IS_SOCKET (GTK_WINDOW (widget)->focus_widget) &&
+	if (GTK_IS_SOCKET (gtk_window_get_focus (GTK_WINDOW (widget))) &&
 	    event->keyval == GDK_F10 &&
 	    (event->state & gtk_accelerator_get_default_mod_mask ()) == GDK_CONTROL_MASK)
 		return gtk_bindings_activate (GTK_OBJECT (widget),
@@ -527,6 +527,13 @@ drop_uri (PanelWidget *panel,
 	if (!icon)
 		icon = g_strdup (fallback_icon);
 
+	/* FIXME: we might get icons like "folder-music" that might not exist in
+	 * the icon theme. This would usually be okay if we could use fallback
+	 * icons (and get "folder" this way). However, this is not possible for
+	 * launchers: this could be an application that uses an icon named
+	 * folder-magic-app, for which we don't want fallbacks. We just want to
+	 * go to hicolor. */
+
 	file = g_file_new_for_uri (uri);
 	buf = g_file_get_parse_name (file);
 	g_object_unref (file);
@@ -718,6 +725,7 @@ move_applet (PanelWidget *panel, int pos, int applet_index)
 {
 	GSList     *applet_list;
 	AppletInfo *info;
+	GtkWidget  *parent;
 
 	applet_list = panel_applet_list_applets ();
 
@@ -729,15 +737,17 @@ move_applet (PanelWidget *panel, int pos, int applet_index)
 	if (pos < 0)
 		pos = 0;
 
+	parent = gtk_widget_get_parent (info->widget);
+
 	if (info != NULL &&
 	    info->widget != NULL &&
-	    info->widget->parent != NULL &&
-	    PANEL_IS_WIDGET (info->widget->parent)) {
+	    parent != NULL &&
+	    PANEL_IS_WIDGET (parent)) {
 		GSList *forb;
 		forb = g_object_get_data (G_OBJECT (info->widget),
 					  PANEL_APPLET_FORBIDDEN_PANELS);
 		if ( ! g_slist_find (forb, panel))
-			panel_widget_reparent (PANEL_WIDGET (info->widget->parent),
+			panel_widget_reparent (PANEL_WIDGET (parent),
 					       panel,
 					       info->widget,
 					       pos);
@@ -887,10 +897,10 @@ panel_check_dnd_target_data (GtkWidget      *widget,
 	    !BUTTON_IS_WIDGET (widget))
 		return FALSE;
 
-	if (!(context->actions & (GDK_ACTION_COPY|GDK_ACTION_MOVE)))
+	if (!(gdk_drag_context_get_actions (context) & (GDK_ACTION_COPY|GDK_ACTION_MOVE)))
 		return FALSE;
 
-	for (l = context->targets; l; l = l->next) {
+	for (l = gdk_drag_context_list_targets (context); l; l = l->next) {
 		GdkAtom atom;
 		guint   info;
 
@@ -965,15 +975,19 @@ panel_check_drop_forbidden (PanelWidget    *panel,
 
 	if (info == TARGET_ICON_INTERNAL ||
 	    info == TARGET_APPLET_INTERNAL) {
-		if (context->actions & GDK_ACTION_MOVE)
+		if (gdk_drag_context_get_actions (context) & GDK_ACTION_MOVE)
 			gdk_drag_status (context, GDK_ACTION_MOVE, time_);
 		else
-			gdk_drag_status (context, context->suggested_action, time_);
+			gdk_drag_status (context,
+					 gdk_drag_context_get_suggested_action (context),
+					 time_);
 
-	} else if (context->actions & GDK_ACTION_COPY)
+	} else if (gdk_drag_context_get_actions (context) & GDK_ACTION_COPY)
 		gdk_drag_status (context, GDK_ACTION_COPY, time_);
 	else
-		gdk_drag_status (context, context->suggested_action, time_);
+		gdk_drag_status (context,
+				 gdk_drag_context_get_suggested_action (context),
+				 time_);
 
 	return TRUE;
 
@@ -1048,25 +1062,28 @@ panel_receive_dnd_data (PanelWidget      *panel,
 			GdkDragContext   *context,
 			guint             time_)
 {
-	gboolean success = FALSE;
+	const guchar *data;
+	gboolean      success = FALSE;
 
 	if (panel_lockdown_get_locked_down ()) {
 		gtk_drag_finish (context, FALSE, FALSE, time_);
 		return;
 	}
 
+	data = gtk_selection_data_get_data (selection_data);
+
 	switch (info) {
 	case TARGET_URL:
-		success = drop_urilist (panel, pos, (char *)selection_data->data);
+		success = drop_urilist (panel, pos, (char *)data);
 		break;
 	case TARGET_NETSCAPE_URL:
-		success = drop_url (panel, pos, (char *)selection_data->data);
+		success = drop_url (panel, pos, (char *)data);
 		break;
 	case TARGET_COLOR:
-		success = set_background_color (panel->toplevel, (guint16 *) selection_data->data);
+		success = set_background_color (panel->toplevel, (guint16 *) data);
 		break;
 	case TARGET_BGIMAGE:
-		success = set_background_image_from_uri (panel->toplevel, (char *) selection_data->data);
+		success = set_background_image_from_uri (panel->toplevel, (char *) data);
 		break;
 	case TARGET_BACKGROUND_RESET:
 		if (panel_profile_is_writable_background_type (panel->toplevel)) {
@@ -1077,28 +1094,28 @@ panel_receive_dnd_data (PanelWidget      *panel,
 		}
 		break;
 	case TARGET_DIRECTORY:
-		success = drop_uri (panel, pos, (char *)selection_data->data,
+		success = drop_uri (panel, pos, (char *)data,
 				    PANEL_ICON_FOLDER);
 		break;
 	case TARGET_APPLET:
-		if (!selection_data->data) {
+		if (!gtk_selection_data_get_data (selection_data)) {
 			gtk_drag_finish (context, FALSE, FALSE, time_);
 			return;
 		}
 		if (panel_profile_id_lists_are_writable ()) {
-			panel_applet_frame_create (panel->toplevel, pos, (char *) selection_data->data);
+			panel_applet_frame_create (panel->toplevel, pos, (char *) data);
 			success = TRUE;
 		} else {
 			success = FALSE;
 		}
 		break;
 	case TARGET_APPLET_INTERNAL:
-		success = drop_internal_applet (panel, pos, (char *)selection_data->data,
-						context->action);
+		success = drop_internal_applet (panel, pos, (char *)data,
+						gdk_drag_context_get_selected_action (context));
 		break;
 	case TARGET_ICON_INTERNAL:
-		success = drop_internal_icon (panel, pos, (char *)selection_data->data,
-					      context->action);
+		success = drop_internal_icon (panel, pos, (char *)data,
+					      gdk_drag_context_get_selected_action (context));
 		break;
 	default:
 		gtk_drag_finish (context, FALSE, FALSE, time_);
@@ -1236,12 +1253,16 @@ panel_screen_from_panel_widget (PanelWidget *panel)
 gboolean
 panel_is_applet_right_stick (GtkWidget *applet)
 {
+	GtkWidget   *parent;
 	PanelWidget *panel_widget;
 
 	g_return_val_if_fail (GTK_IS_WIDGET (applet), FALSE);
-	g_return_val_if_fail (PANEL_IS_WIDGET (applet->parent), FALSE);
 
-	panel_widget = PANEL_WIDGET (applet->parent);
+	parent = gtk_widget_get_parent (applet);
+
+	g_return_val_if_fail (PANEL_IS_WIDGET (parent), FALSE);
+
+	panel_widget = PANEL_WIDGET (parent);
 
 	if (!panel_toplevel_get_expand (panel_widget->toplevel))
 		return FALSE;
